@@ -28,9 +28,11 @@
 #include <Stream.h>
 #include <FontInfo.h>
 #include <PDFDocEncoding.h>
+#include <UGooString.h>
 
 #include "poppler.h"
 #include "poppler-private.h"
+#include "poppler-attachment.h"
 #include "poppler-enums.h"
 
 enum {
@@ -251,11 +253,94 @@ poppler_document_get_page_by_label (PopplerDocument  *document,
   return poppler_document_get_page (document, index);
 }
 
-static gboolean
-has_unicode_marker (GooString *string)
+/**
+ * poppler_document_has_attachments:
+ * @document: A #PopplerDocument
+ * 
+ * Returns %TRUE of @document has any attachments.
+ * 
+ * Return value: %TRUE, if @document has attachments.
+ **/
+gboolean
+poppler_document_has_attachments (PopplerDocument *document)
 {
-  return ((string->getChar (0) & 0xff) == 0xfe &&
-	  (string->getChar (1) & 0xff) == 0xff);
+  Catalog *catalog;
+  int n_files = 0;
+
+  g_return_val_if_fail (POPPLER_IS_DOCUMENT (document), FALSE);
+
+  catalog = document->doc->getCatalog ();
+  if (catalog && catalog->isOk ())
+    {
+      n_files = catalog->numEmbeddedFiles ();
+    }
+
+  return (n_files != 0);
+}
+
+/**
+ * poppler_document_get_attachments:
+ * @document: A #PopplerDocument
+ * 
+ * Returns a #GList containing #PopplerAttachment<!-- -->s.  These attachments
+ * are unowned, and must be unreffed, and the list must be freed with
+ * g_list_free().
+ * 
+ * Return value: a list of available attachments.
+ **/
+GList *
+poppler_document_get_attachments (PopplerDocument *document)
+{
+  Catalog *catalog;
+  int n_files, i;
+  GList *retval = NULL;
+
+  g_return_val_if_fail (POPPLER_IS_DOCUMENT (document), NULL);
+
+  catalog = document->doc->getCatalog ();
+  if (catalog == NULL || ! catalog->isOk ())
+    return NULL;
+
+  n_files = catalog->numEmbeddedFiles ();
+  for (i = 0; i < n_files; i++)
+    {
+      PopplerAttachment *attachment;
+      EmbFile *emb_file;
+
+      emb_file = catalog->embeddedFile (i);
+      attachment = _poppler_attachment_new (document, emb_file);
+
+      retval = g_list_prepend (retval, attachment);
+    }
+  return g_list_reverse (retval);
+}
+
+char *_poppler_goo_string_to_utf8(GooString *s)
+{
+  char *result;
+
+  if (s->hasUnicodeMarker()) {
+    result = g_convert (s->getCString () + 2,
+			s->getLength () - 2,
+			"UTF-8", "UTF-16BE", NULL, NULL, NULL);
+  } else {
+    int len;
+    gunichar *ucs4_temp;
+    int i;
+    
+    len = s->getLength ();
+    ucs4_temp = g_new (gunichar, len + 1);
+    for (i = 0; i < len; ++i) {
+      ucs4_temp[i] = pdfDocEncoding[(unsigned char)s->getChar(i)];
+    }
+    ucs4_temp[i] = 0;
+
+    result = g_ucs4_to_utf8 (ucs4_temp, -1, NULL, NULL, NULL);
+
+    g_free (ucs4_temp);
+  }
+
+  return result;
 }
 
 static void
@@ -272,26 +357,7 @@ info_dict_get_string (Dict *info_dict, const gchar *key, GValue *value)
 
   goo_value = obj.getString ();
 
-  if (has_unicode_marker (goo_value)) {
-    result = g_convert (goo_value->getCString () + 2,
-			goo_value->getLength () - 2,
-			"UTF-8", "UTF-16BE", NULL, NULL, NULL);
-  } else {
-    int len;
-    gunichar *ucs4_temp;
-    int i;
-    
-    len = goo_value->getLength ();
-    ucs4_temp = g_new (gunichar, len + 1);
-    for (i = 0; i < len; ++i) {
-      ucs4_temp[i] = pdfDocEncoding[(unsigned char)goo_value->getChar(i)];
-    }
-    ucs4_temp[i] = 0;
-
-    result = g_ucs4_to_utf8 (ucs4_temp, -1, NULL, NULL, NULL);
-
-    g_free (ucs4_temp);
-  }
+  result = _poppler_goo_string_to_utf8(goo_value);
 
   obj.free ();
 
@@ -318,7 +384,7 @@ info_dict_get_date (Dict *info_dict, const gchar *key, GValue *value)
 
   goo_value = obj.getString (); 
 
-  if (has_unicode_marker (goo_value)) {
+  if (goo_value->hasUnicodeMarker()) {
     date_string = g_convert (goo_value->getCString () + 2,
 			goo_value->getLength () - 2,
 			"UTF-8", "UTF-16BE", NULL, NULL, NULL);		
