@@ -5,9 +5,15 @@
 #include <PDFDoc.h>
 #include <PSOutputDev.h>
 #include <Link.h>
+#include <Gfx.h>
+#include <FontInfo.h>
+#include <TextOutputDev.h>
+#include <Catalog.h>
+
+#include "poppler-attachment.h"
 
 #if defined (HAVE_CAIRO)
-#include <CairoOutputDevImage.h>
+#include <CairoOutputDev.h>
 #elif defined (HAVE_SPLASH)
 #include <SplashOutputDev.h>
 #endif
@@ -18,7 +24,7 @@ struct _PopplerDocument
   PDFDoc *doc;
 
 #if defined (HAVE_CAIRO)
-  CairoOutputDevImage *output_dev;
+  CairoOutputDev *output_dev;
 #elif defined (HAVE_SPLASH)
   SplashOutputDev *output_dev;
 #endif
@@ -28,6 +34,18 @@ struct _PopplerPSFile
 {
   PopplerDocument *document;
   PSOutputDev *out;
+  char *filename;
+  int first_page;
+  int last_page;
+  double paper_width;
+  double paper_height;
+  gboolean duplex;
+};
+
+struct _PopplerFontInfo
+{
+  PopplerDocument *document;
+  FontInfoScanner *scanner;
 };
 
 struct _PopplerPage
@@ -36,7 +54,8 @@ struct _PopplerPage
   PopplerDocument *document;
   Page *page;
   int index;
-  PopplerOrientation orientation;
+  TextOutputDev *text_dev;
+  Gfx *gfx;
 };
 
 PopplerPage   *_poppler_page_new   (PopplerDocument *document,
@@ -46,5 +65,9 @@ PopplerAction *_poppler_action_new (PopplerDocument *document,
 				    LinkAction      *link,
 				    const gchar     *title);
 
+PopplerAttachment *_poppler_attachment_new (PopplerDocument *document,
+					    EmbFile         *file);
+
+char *_poppler_goo_string_to_utf8(GooString *s);
 
 #endif
