@@ -15,6 +15,8 @@
 #pragma interface
 #endif
 
+#include "gtypes.h"
+
 class GooString {
 public:
 
@@ -84,6 +86,8 @@ public:
   int cmp(const char *sA);
   int cmpN(const char *sA, int n);
 
+  GBool hasUnicodeMarker(void);
+
 private:
 
   int length;
