@@ -18,15 +18,13 @@
 
 class CairoFont {
 public:
-  CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib);
+  CairoFont(GfxFont *gfxFont, XRef *xref, FT_Library lib, GBool useCIDs);
   ~CairoFont();
 
   GBool matches(Ref &other);
   cairo_font_face_t *getFontFace(void);
   unsigned long getGlyph(CharCode code, Unicode *u, int uLen);
-  double getSubstitutionCorrection(GfxFont *gfxFont);
 private:
-  int substIdx;
   Ref ref;
   cairo_font_face_t *cairo_font_face;
   FT_Face face;
@@ -56,6 +54,7 @@ public:
 private:
   CairoFont *fontCache[cairoFontCacheSize];
   FT_Library lib;
+  GBool useCIDs;
 };
 
 #endif
