@@ -35,6 +35,7 @@
 #include "Error.h"
 #include "Gfx.h"
 #include "ProfileData.h"
+#include "UGooString.h"
 
 // the MSVC math.h doesn't define this
 #ifndef M_PI
@@ -3525,6 +3526,7 @@ Stream *Gfx::buildImageStream() {
 	break;
       }
       dict.dictAdd(key, &obj);
+      gfree(key);
     }
     parser->getObj(&obj);
   }
@@ -3590,9 +3592,16 @@ void Gfx::opBeginMarkedContent(Object args[], int numArgs) {
     printf("\n");
     fflush(stdout);
   }
+
+  if(numArgs == 2) {
+    out->beginMarkedContent(args[0].getName(),args[1].getDict());
+  } else {
+    out->beginMarkedContent(args[0].getName());
+  }
 }
 
 void Gfx::opEndMarkedContent(Object args[], int numArgs) {
+  out->endMarkedContent();
 }
 
 void Gfx::opMarkPoint(Object args[], int numArgs) {
@@ -3603,6 +3612,13 @@ void Gfx::opMarkPoint(Object args[], int numArgs) {
     printf("\n");
     fflush(stdout);
   }
+
+  if(numArgs == 2) {
+    out->markPoint(args[0].getName(),args[1].getDict());
+  } else {
+    out->markPoint(args[0].getName());
+  }
+
 }
 
 //------------------------------------------------------------------------
