@@ -26,12 +26,12 @@
 #include <Catalog.h>
 #include <ErrorCodes.h>
 #include <ArthurOutputDev.h>
-#include <Private.h>
 #include <SplashOutputDev.h>
 #include <TextOutputDev.h>
 #include <splash/SplashBitmap.h>
 
 #include "poppler-private.h"
+#include "poppler-page-transition-private.h"
 
 namespace Poppler {
 
